@@ -46,7 +46,9 @@
 #include "../int/term_title.h"
 #include "../io/iop.h"
 #include "../modes/dialogs/msg_dialog.h"
+#include "../ui/color_scheme.h"
 #include "../ui/statusbar.h"
+#include "../ui/tabs.h"
 #include "../ui/ui.h"
 #include "../utils/env.h"
 #include "../utils/file_streams.h"
@@ -55,6 +57,7 @@
 #include "../utils/macros.h"
 #include "../utils/str.h"
 #include "../utils/path.h"
+#include "../utils/string_array.h"
 #include "../utils/utils.h"
 #include "../cmd_core.h"
 #include "../filelist.h"
@@ -102,12 +105,14 @@ static void copy_help_file(void);
 static void create_scripts_dir(void);
 static void copy_rc_file(void);
 static void add_default_marks(void);
-static int source_file_internal(FILE *fp, const char filename[]);
+static int source_file_internal(strlist_t lines, const char filename[]);
 static void show_sourcing_error(const char filename[], int line_num);
 
 void
 cfg_init(void)
 {
+	cfg.session = NULL;
+
 	cfg.history_len = 15;
 
 	cfg.auto_execute = 0;
@@ -136,6 +141,8 @@ cfg_init(void)
 	cfg.smart_case = 0;
 	cfg.hl_search = 1;
 	cfg.vifm_info = VINFO_MARKS | VINFO_BOOKMARKS;
+	cfg.session_options = VINFO_TUI | VINFO_STATE | VINFO_TABS | VINFO_SAVEDIRS
+	                    | VINFO_DHISTORY;
 	cfg.scroll_off = 0;
 	cfg.gdefault = 0;
 	cfg.scroll_bind = 0;
@@ -151,7 +158,7 @@ cfg_init(void)
 	cfg.lines = INT_MIN;
 	cfg.columns = INT_MIN;
 
-	cfg.dot_dirs = DD_NONROOT_PARENT;
+	cfg.dot_dirs = DD_NONROOT_PARENT | DD_TREE_LEAFS_PARENT;
 
 	cfg.filter_inverted_by_default = 1;
 
@@ -176,6 +183,7 @@ cfg_init(void)
 	cfg.extra_padding = 1;
 	cfg.side_borders_visible = 1;
 	cfg.use_unicode_characters = 0;
+	cfg.flexible_splitter = 1;
 	cfg.display_statusline = 1;
 
 	cfg.border_filler = strdup(" ");
@@ -220,6 +228,9 @@ cfg_init(void)
 
 	cfg.pane_tabs = 0;
 	cfg.show_tab_line = STL_MULTIPLE;
+	cfg.tab_prefix = strdup("[%N:");
+	cfg.tab_label = strdup("");
+	cfg.tab_suffix = strdup("]");
 
 	cfg.auto_ch_pos = 1;
 	cfg.ch_pos_on = CHPOS_STARTUP | CHPOS_DIRMARK | CHPOS_ENTER;
@@ -487,7 +498,7 @@ try_exe_directory_for_vifmrc(void)
 	LOG_FUNC_ENTER;
 
 	char exe_dir[PATH_MAX + 1];
-	char vifmrc[PATH_MAX + 1];
+	char vifmrc[PATH_MAX + 8];
 
 	if(get_exe_dir(exe_dir, sizeof(exe_dir)) != 0)
 	{
@@ -582,6 +593,7 @@ setup_dirs(void)
 
 	copy_help_file();
 	copy_rc_file();
+	cs_write();
 
 	/* Ensure that just copied sample vifmrc file is used right away. */
 	snprintf(rc_file, sizeof(rc_file), "%s/" VIFMRC, cfg.config_dir);
@@ -619,7 +631,7 @@ static void
 create_scripts_dir(void)
 {
 	char scripts[PATH_MAX + 16];
-	char readme[PATH_MAX + 1];
+	char readme[PATH_MAX + 32];
 	FILE *fp;
 
 	snprintf(scripts, sizeof(scripts), "%s/" SCRIPTS_DIR, cfg.config_dir);
@@ -671,8 +683,8 @@ add_default_marks(void)
 {
 	LOG_FUNC_ENTER;
 
-	set_user_mark('H', cfg.home_dir, NO_MARK_FILE);
-	set_user_mark('z', cfg.config_dir, NO_MARK_FILE);
+	marks_set_user(curr_view, 'H', cfg.home_dir, NO_MARK_FILE);
+	marks_set_user(curr_view, 'z', cfg.config_dir, NO_MARK_FILE);
 }
 
 void
@@ -685,8 +697,11 @@ cfg_load(void)
 	 * views. */
 	curr_stats.global_local_settings = 1;
 
-	/* Try to load global configuration. */
-	(void)cfg_source_file(PACKAGE_SYSCONF_DIR "/" VIFMRC);
+	if(!vifm_testing())
+	{
+		/* Try to load global configuration. */
+		(void)cfg_source_file(PACKAGE_SYSCONF_DIR "/" VIFMRC);
+	}
 
 	/* Try to load local configuration. */
 	rc = env_get(MYVIFMRC_EV);
@@ -703,64 +718,68 @@ cfg_source_file(const char filename[])
 {
 	/* TODO: maybe move this to commands.c or separate unit eventually. */
 
-	FILE *fp;
-	int result;
-	SourcingState sourcing_state;
-
-	if((fp = os_fopen(filename, "r")) == NULL)
+	/* Binary mode is important on Windows. */
+	FILE *fp = os_fopen(filename, "rb");
+	if(fp == NULL)
 	{
 		return 1;
 	}
 
-	sourcing_state = curr_stats.sourcing_state;
+	strlist_t lines;
+	lines.items = read_file_lines(fp, &lines.nitems);
+	fclose(fp);
+
+	SourcingState sourcing_state = curr_stats.sourcing_state;
 	curr_stats.sourcing_state = SOURCING_PROCESSING;
 
-	result = source_file_internal(fp, filename);
+	int result = source_file_internal(lines, filename);
 
 	curr_stats.sourcing_state = sourcing_state;
 
-	fclose(fp);
+	free_string_array(lines.items, lines.nitems);
 	return result;
 }
 
 /* Returns non-zero on error. */
 static int
-source_file_internal(FILE *fp, const char filename[])
+source_file_internal(strlist_t lines, const char filename[])
 {
-	char line[MAX_VIFMRC_LINE_LEN + 1];
-	char *next_line = NULL;
-	int line_num;
-	int encoutered_errors = 0;
-
-	if(fgets(line, sizeof(line), fp) == NULL)
+	if(lines.nitems == 0)
 	{
-		/* File is empty. */
 		return 0;
 	}
-	chomp(line);
 
 	commands_scope_start();
 
-	line_num = 1;
+	int encoutered_errors = 0;
+
+	char line[MAX_VIFMRC_LINE_LEN + 1];
+	line[0] = '\0';
+
+	int line_num = 0;
+	int next_line_num = 0;
 	for(;;)
 	{
-		char *p;
-		int line_num_delta = 0;
-
-		while((p = next_line = read_line(fp, next_line)) != NULL)
+		char *next_line = NULL;
+		while(next_line_num < lines.nitems)
 		{
-			line_num_delta++;
-			p = skip_whitespace(p);
-			if(*p == '"')
+			next_line = skip_whitespace(lines.items[next_line_num++]);
+			if(*next_line == '"')
+			{
 				continue;
-			else if(*p == '\\')
-				strncat(line, p + 1, sizeof(line) - strlen(line) - 1);
+			}
+			else if(*next_line == '\\')
+			{
+				strncat(line, next_line + 1, sizeof(line) - strlen(line) - 1);
+				next_line = NULL;
+			}
 			else
+			{
 				break;
+			}
 		}
 
-		/* Clear statusbar message. */
-		ui_sb_msg("");
+		ui_sb_clear();
 
 		if(exec_commands(line, curr_view, CIT_COMMAND) < 0)
 		{
@@ -770,7 +789,7 @@ source_file_internal(FILE *fp, const char filename[])
 		if(curr_stats.sourcing_state == SOURCING_FINISHING)
 			break;
 
-		if(p == NULL)
+		if(next_line == NULL)
 		{
 			/* Artificially increment line number to simulate as if all that happens
 			 * after the loop relates to something past end of the file. */
@@ -778,14 +797,12 @@ source_file_internal(FILE *fp, const char filename[])
 			break;
 		}
 
-		copy_str(line, sizeof(line), p);
-		line_num += line_num_delta;
+		copy_str(line, sizeof(line), next_line);
+		line_num = next_line_num;
 	}
 
-	free(next_line);
+	ui_sb_clear();
 
-	/* Clear statusbar message. */
-	ui_sb_msg("");
 	if(commands_scope_finish() != 0)
 	{
 		show_sourcing_error(filename, line_num);
@@ -839,21 +856,45 @@ cfg_get_vicmd(int *bg)
 	}
 }
 
+void
+cfg_clear_histories(int clear_dhistory)
+{
+	if(clear_dhistory)
+	{
+		cfg_resize_histories(0);
+	}
+	else
+	{
+		hists_resize(0);
+		cfg.history_len = 0;
+		/* Directory histories keep their data, which will be truncated or updated
+		 * later. */
+	}
+}
+
 void
 cfg_resize_histories(int new_size)
 {
-	const int old_size = MAX(cfg.history_len, 0);
-
 	hists_resize(new_size);
-	flist_hist_resize(&lwin, new_size);
-	flist_hist_resize(&rwin, new_size);
 
+	int i;
+	tab_info_t tab_info;
+	for(i = 0; tabs_enum_all(i, &tab_info); ++i)
+	{
+		flist_hist_resize(tab_info.view, new_size);
+	}
+
+	const int old_size = cfg.history_len;
 	cfg.history_len = new_size;
 
-	if(old_size == 0 && new_size > 0)
+	if(old_size <= 0 && new_size > 0)
 	{
-		flist_hist_save(&lwin, NULL, NULL, -1);
-		flist_hist_save(&rwin, NULL, NULL, -1);
+		int i;
+		tab_info_t tab_info;
+		for(i = 0; tabs_enum_all(i, &tab_info); ++i)
+		{
+			flist_hist_save(tab_info.view);
+		}
 	}
 }
 
