@@ -20,8 +20,6 @@
 #include "cmd_completion.h"
 
 #ifdef _WIN32
-#define REQUIRED_WINVER 0x0500 /* To get GetFileSizeEx() function. */
-#include "utils/windefs.h"
 #include <windows.h>
 #include <lm.h>
 #endif
@@ -42,6 +40,7 @@
                        strrchr() */
 
 #include "cfg/config.h"
+#include "cfg/info.h"
 #include "compat/dtype.h"
 #include "compat/fs_limits.h"
 #include "compat/os.h"
@@ -272,6 +271,13 @@ complete_args(int id, const cmd_info_t *cmd_info, int arg_pos, void *extra_arg)
 	{
 		cs_complete(arg);
 	}
+	else if(id == COM_SESSION || id == COM_DELSESSION)
+	{
+		if(earg_num(argc, args) <= 1 && !cmd_info->emark && !cmd_info->qmark)
+		{
+			sessions_complete(arg);
+		}
+	}
 	else if(id == COM_WINCMD)
 	{
 		if(earg_num(argc, args) <= 1)
