@@ -54,7 +54,7 @@
 #define BLOCK_SIZE (32*1024)
 
 /* Amount of data to hash for coarse comparison. */
-#define PREFIX_SIZE (256*1024)
+#define PREFIX_SIZE (4*1024)
 
 /* Entry in singly-bounded list of files that have matched fingerprints. */
 typedef struct compare_record_t
@@ -105,14 +105,13 @@ compare_two_panes(CompareType ct, ListType lt, int group_paths, int skip_empty)
 	entries_t curr, other;
 
 	trie_t *const trie = trie_create();
-	ui_cancellation_reset();
-	ui_cancellation_enable();
+	ui_cancellation_push_on();
 
 	curr = make_diff_list(trie, curr_view, &next_id, ct, skip_empty, 0);
 	other = make_diff_list(trie, other_view, &next_id, ct, skip_empty,
 			lt == LT_DUPS);
 
-	ui_cancellation_disable();
+	ui_cancellation_pop();
 	trie_free_with_data(trie, &free_compare_records);
 
 	/* Clear progress message displayed by make_diff_list(). */
@@ -387,12 +386,11 @@ compare_one_pane(view_t *view, CompareType ct, ListType lt, int skip_empty)
 	entries_t curr;
 
 	trie_t *trie = trie_create();
-	ui_cancellation_reset();
-	ui_cancellation_enable();
+	ui_cancellation_push_on();
 
 	curr = make_diff_list(trie, view, &next_id, ct, skip_empty, 0);
 
-	ui_cancellation_disable();
+	ui_cancellation_pop();
 	trie_free_with_data(trie, &free_compare_records);
 
 	/* Clear progress message displayed by make_diff_list(). */
@@ -455,7 +453,7 @@ compare_one_pane(view_t *view, CompareType ct, ListType lt, int skip_empty)
 	 * this. */
 	if(other->custom.type == CV_DIFF)
 	{
-		cd_updir(other, 1);
+		rn_leave(other, 1);
 	}
 
 	view->list_pos = 0;
@@ -612,8 +610,7 @@ append_valid_nodes(const char name[], int valid, const void *parent_data,
 	{
 		char full_path[PATH_MAX + 1];
 		get_full_path_of(entry, sizeof(full_path), full_path);
-		list->nitems = add_to_string_array(&list->items, list->nitems, 1,
-				full_path);
+		list->nitems = add_to_string_array(&list->items, list->nitems, full_path);
 	}
 	return 0;
 }
