@@ -17,7 +17,7 @@
  */
 
 /* WARNING: code implementing execution of keys deserves a prize in nomination
- *          "insanely perplexed control flow".  You have been warned.
+ *          "insanely perplex control flow".  You have been warned.
  * TODO: seriously, indirect recursion and jumps between routines is a horrible
  *       mess, even not touching tree representation this should be possible to
  *       rewrite with master loop that simply goes through the string in
@@ -86,8 +86,18 @@ static default_handler *def_handlers;
 static size_t counter;
 /* Main external functions enter recursion level. */
 static size_t enters_counter;
+/* Current (when inside) or previous (when outside) sequence number of entering
+ * input resolution machinery. */
+static size_t enter_seq;
 /* Shows whether a mapping handler is being executed at the moment. */
 static int inside_mapping;
+/* Current (when inside mapping) or previous (when outside) sequence number of a
+ * mapping state.  It gets updated when a mapping handling is started within
+ * current input resolution request (basically unit was entered from outside
+ * and hasn't left yet). */
+static int mapping_state;
+/* Value of enter_seq variable last seen on processing a mapping. */
+static size_t mapping_enter_seq;
 /* User-provided callback for silencing UI. */
 static vle_silence_func silence_ui;
 
@@ -282,11 +292,12 @@ static int
 execute_keys_general_wrapper(const wchar_t keys[], int timed_out, int mapped,
 		int no_remap)
 {
-	int result;
-
-	enters_counter++;
-	result = execute_keys_general(keys, timed_out, mapped, no_remap);
-	enters_counter--;
+	if(++enters_counter == 1)
+	{
+		enter_seq = (enter_seq == INT_MAX ? 1 : enter_seq + 1);
+	}
+	int result = execute_keys_general(keys, timed_out, mapped, no_remap);
+	--enters_counter;
 
 	return result;
 }
@@ -419,8 +430,19 @@ dispatch_keys_at_root(const wchar_t keys[], keys_info_t *keys_info,
 					contains_chain(&builtin_cmds_root[vle_mode_get()], keys_start, keys);
 			result = execute_next_keys(curr, curr->type == USER_CMD ? keys : L"",
 					&key_info, keys_info, has_duplicate, no_remap);
+
 			if(curr->type == USER_CMD)
-				return result;
+			{
+				/* We've at least attempted to execute a user mapping.  Maybe it did
+				 * nothing, but it's also possible that it executed something and failed
+				 * afterwards.  Either way, finishing processing successfully is the
+				 * best course of action.  We either really did succeed or we need to
+				 * avoid any further processing anyway (trying to interpret LHS of the
+				 * mapping in a different way would be a mistake and waiting of any kind
+				 * within a mapping can only be ignored). */
+				return 0;
+			}
+
 			if(IS_KEYS_RET_CODE(result))
 			{
 				return (result == KEYS_WAIT_SHORT) ? KEYS_UNKNOWN : result;
@@ -787,6 +809,7 @@ execute_after_remapping(const wchar_t rhs[], const wchar_t left_keys[],
 		}
 		if(key_info.count != NO_COUNT_GIVEN)
 		{
+			/* XXX: this can insert thousand separators in some locales... */
 			vifm_swprintf(buf + wcslen(buf), ARRAY_LEN(buf) - wcslen(buf), L"%d",
 					key_info.count);
 		}
@@ -798,6 +821,9 @@ execute_after_remapping(const wchar_t rhs[], const wchar_t left_keys[],
 			init_keys_info(&keys_info, 1);
 		}
 
+		/* XXX: keys_info.mapped now covers both RHS of a mapping and the rest of
+		 *      the keys (`left_keys`)!  This is bad. */
+
 		enter_chunk(curr);
 		result = dispatch_keys(buf, &keys_info, curr->no_remap, NO_COUNT_GIVEN);
 		leave_chunk(curr);
@@ -1181,9 +1207,9 @@ is_recursive(void)
 }
 
 int
-vle_keys_inside_mapping(void)
+vle_keys_mapping_state(void)
 {
-	return (inside_mapping != 0);
+	return (inside_mapping ? mapping_state : 0);
 }
 
 /* Executes handler for a mapping, if any.  Error or success code is
@@ -1206,8 +1232,17 @@ execute_mapping_handler(const key_conf_t *info, key_info_t key_info,
 static void
 pre_execute_mapping_handler(const keys_info_t *keys_info)
 {
-	inside_mapping += keys_info->mapped != 0;
-	assert(inside_mapping >= 0 && "Calls to pre/post funcs should be balanced");
+	if(keys_info->mapped)
+	{
+		++inside_mapping;
+		assert(inside_mapping >= 0 && "Calls to pre/post funcs should be balanced");
+
+		if(inside_mapping == 1 && mapping_enter_seq != enter_seq)
+		{
+			mapping_state = (mapping_state == INT_MAX ? 1 : mapping_state + 1);
+			mapping_enter_seq = enter_seq;
+		}
+	}
 }
 
 /* Post-execution of a mapping handler callback. */
@@ -1252,11 +1287,23 @@ keys_suggest(const key_chunk_t *root, const wchar_t keys[],
 			}
 		}
 
-		/* Go to the next character if a match found. */
+		/* Go to the next character if a match is found. */
 		if(p != NULL && p->key == *keys)
 		{
 			++keys;
 			curr = p;
+
+			/* Branch into matching selectors at every step to handle situation when
+			 * multichar selector and multichar command continuation are available at
+			 * the same time.. */
+			if(curr->type == BUILTIN_WAIT_POINT &&
+					curr->conf.followed == FOLLOWED_BY_SELECTOR)
+			{
+				/* Suggest selectors. */
+				keys_suggest(&selectors_root[vle_mode_get()], keys, L"sel: ", cb,
+						custom_only, fold_subkeys);
+			}
+
 			continue;
 		}
 
@@ -1300,24 +1347,28 @@ keys_suggest(const key_chunk_t *root, const wchar_t keys[],
 
 	if(!custom_only && *keys == L'\0')
 	{
+		if(curr->type == USER_CMD)
+		{
+			if(!curr->no_remap)
+			{
+				keys_suggest(&user_cmds_root[vle_mode_get()], curr->conf.data.cmd,
+						prefix, cb, custom_only, fold_subkeys);
+			}
+			keys_suggest(&builtin_cmds_root[vle_mode_get()], curr->conf.data.cmd,
+					prefix, cb, custom_only, fold_subkeys);
+			return;
+		}
+
 		suggest_children(curr, prefix, cb, fold_subkeys);
 	}
 
-	if(curr->type == BUILTIN_WAIT_POINT)
+	if(curr->type == BUILTIN_WAIT_POINT &&
+			curr->conf.followed == FOLLOWED_BY_MULTIKEY)
 	{
-		if(curr->conf.followed == FOLLOWED_BY_SELECTOR)
+		/* Invoke optional external function to provide suggestions. */
+		if(curr->conf.suggest != NULL)
 		{
-			/* Suggest selectors. */
-			keys_suggest(&selectors_root[vle_mode_get()], keys, L"sel: ", cb,
-					custom_only, fold_subkeys);
-		}
-		else if(curr->conf.followed == FOLLOWED_BY_MULTIKEY)
-		{
-			/* Invoke optional external function to provide suggestions. */
-			if(curr->conf.suggest != NULL)
-			{
-				curr->conf.suggest(cb);
-			}
+			curr->conf.suggest(cb);
 		}
 	}
 }
