@@ -17,8 +17,6 @@
  */
 
 #ifdef _WIN32
-#define REQUIRED_WINVER 0x0600
-#include "../utils/windefs.h"
 #include <windows.h>
 #endif
 
@@ -292,8 +290,6 @@ iop_cp_internal(io_args_t *args)
 
 	char block[BLOCK_SIZE];
 	FILE *in, *out;
-	/* Suppress possible false-positive compiler warning. */
-	size_t nread = (size_t)-1;
 	int error;
 	int cloned;
 	struct stat src_st;
@@ -545,6 +541,8 @@ iop_cp_internal(io_args_t *args)
 
 	if(!error && !cloned)
 	{
+		/* Suppress possible false-positive compiler warning. */
+		size_t nread = (size_t)-1;
 		while((nread = fread(&block, 1, sizeof(block), in)) != 0U)
 		{
 			if(io_cancelled(args))
@@ -731,22 +729,25 @@ iop_ln_internal(io_args_t *args)
 	char *escaped_path, *escaped_target;
 	char base_dir[PATH_MAX + 2];
 
-	if(!overwrite && path_exists(target, DEREF))
+	if(path_exists(target, NODEREF))
 	{
-		(void)ioe_errlst_append(&args->result.errors, target, EEXIST,
-				"Destination path already exists");
-		return -1;
+		if(!overwrite)
+		{
+			(void)ioe_errlst_append(&args->result.errors, target, EEXIST,
+					"Destination path already exists");
+			return -1;
+		}
+
+		if(!is_symlink(target))
+		{
+			(void)ioe_errlst_append(&args->result.errors, target, IO_ERR_UNKNOWN,
+					"Target is not a symbolic link");
+			return -1;
+		}
 	}
 
-	if(overwrite && !is_symlink(target))
-	{
-		(void)ioe_errlst_append(&args->result.errors, target, IO_ERR_UNKNOWN,
-				"Target is not a symbolic link");
-		return -1;
-	}
-
-	escaped_path = shell_like_escape(path, 0);
-	escaped_target = shell_like_escape(target, 0);
+	escaped_path = strdup(enclose_in_dquotes(path));
+	escaped_target = strdup(enclose_in_dquotes(target));
 	if(escaped_path == NULL || escaped_target == NULL)
 	{
 		(void)ioe_errlst_append(&args->result.errors, target, IO_ERR_UNKNOWN,
