@@ -114,7 +114,7 @@ typedef struct
 	int old_top;              /* for search_mode */
 	int old_pos;              /* for search_mode */
 	int line_edited;          /* Cache for whether input line changed flag. */
-	int entered_by_mapping;   /* The mode was entered by a mapping. */
+	int enter_mapping_state;  /* The mapping state at entering the mode. */
 	int expanding_abbrev;     /* Abbreviation expansion is in progress. */
 	PromptState state;        /* Prompt state with regard to current input. */
 }
@@ -279,10 +279,10 @@ static keys_add_info_t builtin_cmds[] = {
 	{WK_ESC WK_f,     {{&cmd_meta_f}, .descr = "move cursor to next word"}},
 	{WK_ESC WK_DOT,   {{&cmd_meta_dot}, .descr = "start/continue last arg completion"}},
 #else
-	{{ALT_B},         {{&cmd_meta_b}, .descr = "move cursor to previous word"}},
-	{{ALT_D},         {{&cmd_meta_d}, .descr = "remove next word"}},
-	{{ALT_F},         {{&cmd_meta_f}, .descr = "move cursor to next word"}},
-	{{ALT_PERIOD},    {{&cmd_meta_dot}, .descr = "start/continue last arg completion"}},
+	{{K(ALT_B)},      {{&cmd_meta_b}, .descr = "move cursor to previous word"}},
+	{{K(ALT_D)},      {{&cmd_meta_d}, .descr = "remove next word"}},
+	{{K(ALT_F)},      {{&cmd_meta_f}, .descr = "move cursor to next word"}},
+	{{K(ALT_PERIOD)}, {{&cmd_meta_dot}, .descr = "start/continue last arg completion"}},
 #endif
 #ifdef ENABLE_EXTENDED_KEYS
 	{{K(KEY_BACKSPACE)}, {{&cmd_ctrl_h}, .descr = "remove char to the left"}},
@@ -298,7 +298,7 @@ static keys_add_info_t builtin_cmds[] = {
 };
 
 void
-init_cmdline_mode(void)
+modcline_init(void)
 {
 	int ret_code;
 
@@ -456,7 +456,7 @@ input_line_changed(void)
 	}
 	else if(prev_mode == MENU_MODE)
 	{
-		menu_full_redraw();
+		modmenu_full_redraw();
 	}
 
 	/* Hardware cursor is moved on the screen only on refresh, so refresh status
@@ -503,12 +503,12 @@ handle_nonempty_input(void)
 
 		case CLS_BSEARCH: backward = 1; /* Fall through. */
 		case CLS_FSEARCH:
-			result = find_npattern(curr_view, mbinput, backward, 0);
+			result = modnorm_find(curr_view, mbinput, backward, 0);
 			update_state(result, curr_view->matches);
 			break;
 		case CLS_VBSEARCH: backward = 1; /* Fall through. */
 		case CLS_VFSEARCH:
-			result = find_vpattern(curr_view, mbinput, backward, 0);
+			result = modvis_find(curr_view, mbinput, backward, 0);
 			update_state(result, curr_view->matches);
 			break;
 		case CLS_MENU_FSEARCH:
@@ -585,7 +585,7 @@ wcsins(wchar_t src[], const wchar_t ins[], int pos)
 }
 
 void
-enter_cmdline_mode(CmdLineSubmode cl_sub_mode, const char cmd[], void *ptr)
+modcline_enter(CmdLineSubmode cl_sub_mode, const char cmd[], void *ptr)
 {
 	wchar_t *wcmd;
 	const wchar_t *wprompt;
@@ -636,7 +636,7 @@ enter_cmdline_mode(CmdLineSubmode cl_sub_mode, const char cmd[], void *ptr)
 }
 
 void
-enter_prompt_mode(const char prompt[], const char cmd[], prompt_cb cb,
+modcline_prompt(const char prompt[], const char cmd[], prompt_cb cb,
 		complete_cmd_func complete, int allow_ee)
 {
 	wchar_t *wprompt;
@@ -662,11 +662,11 @@ enter_prompt_mode(const char prompt[], const char cmd[], prompt_cb cb,
 }
 
 void
-redraw_cmdline(void)
+modcline_redraw(void)
 {
 	if(prev_mode == MENU_MODE)
 	{
-		menu_full_redraw();
+		modmenu_full_redraw();
 	}
 	else
 	{
@@ -718,7 +718,7 @@ prepare_cmdline_mode(const wchar_t prompt[], const wchar_t cmd[],
 	input_stat.search_mode = 0;
 	input_stat.dot_pos = -1;
 	input_stat.line_edited = 0;
-	input_stat.entered_by_mapping = vle_keys_inside_mapping();
+	input_stat.enter_mapping_state = vle_keys_mapping_state();
 	input_stat.state = PS_NORMAL;
 
 	if((is_forward_search(sub_mode) || is_backward_search(sub_mode)) &&
@@ -760,7 +760,7 @@ save_view_port(void)
 	}
 	else
 	{
-		menu_save_pos();
+		modmenu_save_pos();
 	}
 }
 
@@ -770,7 +770,7 @@ set_view_port(void)
 {
 	if(prev_mode == MENU_MODE)
 	{
-		menu_restore_pos();
+		modmenu_restore_pos();
 		return;
 	}
 
@@ -787,7 +787,7 @@ set_view_port(void)
 
 	if(prev_mode == VISUAL_MODE)
 	{
-		update_visual_mode();
+		modvis_update();
 	}
 }
 
@@ -813,12 +813,12 @@ leave_cmdline_mode(void)
 	wresize(status_bar, 1, getmaxx(stdscr) - FIELDS_WIDTH());
 	if(multiline_status_bar)
 	{
-		curr_stats.need_update = UT_REDRAW;
+		stats_redraw_later();
 		mvwin(status_bar, getmaxy(stdscr) - 1, 0);
 		if(prev_mode == MENU_MODE)
 		{
 			wresize(menu_win, getmaxy(stdscr) - 1, getmaxx(stdscr));
-			menu_partial_redraw();
+			modmenu_partial_redraw();
 		}
 	}
 
@@ -880,8 +880,8 @@ cmd_ctrl_c(key_info_t key_info, keys_info_t *keys_info)
 	{
 		if(!input_stat.search_mode)
 		{
-			leave_visual_mode(curr_stats.save_msg, 1, 1);
-			fpos_set_pos(curr_view, check_mark_directory(curr_view, '<'));
+			modvis_leave(curr_stats.save_msg, 1, 1);
+			fpos_set_pos(curr_view, marks_find_in_view(curr_view, '<'));
 		}
 	}
 	if(sub_mode == CLS_COMMAND)
@@ -1283,7 +1283,9 @@ cmd_return(key_info_t key_info, keys_info_t *keys_info)
 
 	if(prev_mode == VISUAL_MODE && sub_mode != CLS_VFSEARCH &&
 			sub_mode != CLS_VBSEARCH)
-		leave_visual_mode(curr_stats.save_msg, 1, 0);
+	{
+		modvis_leave(curr_stats.save_msg, 1, 0);
+	}
 
 	save_input_to_history(keys_info, input);
 
@@ -1348,7 +1350,7 @@ cmd_return(key_info_t key_info, keys_info_t *keys_info)
 				}
 			case CLS_MENU_FSEARCH:
 			case CLS_MENU_BSEARCH:
-				curr_stats.need_update = UT_FULL;
+				stats_refresh_later();
 				curr_stats.save_msg = menus_search(pattern, sub_mode_ptr, 1);
 				break;
 
@@ -1480,7 +1482,8 @@ save_input_to_history(const keys_info_t *keys_info, const char input[])
 	}
 	else if(sub_mode == CLS_COMMAND)
 	{
-		const int mapped_input = input_stat.entered_by_mapping && keys_info->mapped;
+		const int mapped_input = input_stat.enter_mapping_state != 0 &&
+			vle_keys_mapping_state() == input_stat.enter_mapping_state;
 		const int ignore_input = mapped_input || keys_info->recursive;
 		if(!ignore_input)
 		{
@@ -1638,7 +1641,7 @@ hist_next(line_stats_t *stat, const hist_t *hist, size_t len)
 		int len = stat->hist_search_len;
 		while(--pos >= 0)
 		{
-			wchar_t *const buf = to_wide(hist->items[pos]);
+			wchar_t *const buf = to_wide(hist->items[pos].text);
 			if(wcsncmp(stat->line, buf, len) == 0)
 			{
 				free(buf);
@@ -1654,7 +1657,7 @@ hist_next(line_stats_t *stat, const hist_t *hist, size_t len)
 		stat->cmd_pos = pos;
 	}
 
-	(void)replace_input_line(stat, hist->items[stat->cmd_pos]);
+	(void)replace_input_line(stat, hist->items[stat->cmd_pos].text);
 
 	update_cmdline(stat);
 
@@ -2074,24 +2077,18 @@ remove_previous_dot_completion(void)
 static wchar_t *
 next_dot_completion(void)
 {
-	size_t len;
-	char *last;
-	wchar_t *wide;
+	if(input_stat.dot_pos >= curr_stats.cmd_hist.size)
+	{
+		return vifm_wcsdup(L"");
+	}
 
-	if(input_stat.dot_pos <= curr_stats.cmd_hist.pos)
-	{
-		last = vle_cmds_last_arg(curr_stats.cmd_hist.items[input_stat.dot_pos++], 1,
-				&len);
-	}
-	else
-	{
-		last = "";
-		len = 0;
-	}
-	last = strdup(last);
-	last[len] = '\0';
-	wide = to_wide(last);
-	free(last);
+	size_t len;
+	const char *last_entry = curr_stats.cmd_hist.items[input_stat.dot_pos++].text;
+	const char *last_arg_pos = vle_cmds_last_arg(last_entry, 1, &len);
+
+	char *last_arg = format_str("%.*s", (int)len, last_arg_pos);
+	wchar_t *wide = to_wide(last_arg);
+	free(last_arg);
 
 	return wide;
 }
@@ -2317,7 +2314,7 @@ hist_prev(line_stats_t *stat, const hist_t *hist, size_t len)
 
 	if(stat->history_search != HIST_SEARCH)
 	{
-		if(stat->cmd_pos == hist->pos)
+		if(stat->cmd_pos == hist->size - 1)
 		{
 			return;
 		}
@@ -2329,9 +2326,9 @@ hist_prev(line_stats_t *stat, const hist_t *hist, size_t len)
 		 * string.  Initially cmd_pos is -1, no need to check anything if history
 		 * contains only one element as even if it's equal input line won't be
 		 * changed. */
-		if(stat->cmd_pos == 0 && hist->pos != 0)
+		if(stat->cmd_pos == 0 && hist->size != 1)
 		{
-			wchar_t *const wide_item = to_wide(hist->items[0]);
+			wchar_t *const wide_item = to_wide(hist->items[0].text);
 			if(wcscmp(stat->line, wide_item) == 0)
 			{
 				++stat->cmd_pos;
@@ -2343,9 +2340,9 @@ hist_prev(line_stats_t *stat, const hist_t *hist, size_t len)
 	{
 		int pos = stat->cmd_pos;
 		int len = stat->hist_search_len;
-		while(++pos <= hist->pos)
+		while(++pos < hist->size)
 		{
-			wchar_t *const wide_item = to_wide(hist->items[pos]);
+			wchar_t *const wide_item = to_wide(hist->items[pos].text);
 			if(wcsncmp(stat->line, wide_item, len) == 0)
 			{
 				free(wide_item);
@@ -2353,12 +2350,12 @@ hist_prev(line_stats_t *stat, const hist_t *hist, size_t len)
 			}
 			free(wide_item);
 		}
-		if(pos > hist->pos)
+		if(pos >= hist->size)
 			return;
 		stat->cmd_pos = pos;
 	}
 
-	(void)replace_input_line(stat, hist->items[stat->cmd_pos]);
+	(void)replace_input_line(stat, hist->items[stat->cmd_pos].text);
 
 	update_cmdline(stat);
 
@@ -2484,7 +2481,7 @@ update_cmdline_size(void)
 	else
 	{
 		wresize(menu_win, getmaxy(stdscr) - required_height, getmaxx(stdscr));
-		menu_partial_redraw();
+		modmenu_partial_redraw();
 	}
 }
 
@@ -2660,7 +2657,7 @@ stop_regular_completion(void)
 	{
 		if(sub_mode == CLS_MENU_COMMAND)
 		{
-			menu_full_redraw();
+			modmenu_full_redraw();
 		}
 		else
 		{
