@@ -48,22 +48,21 @@ typedef void (*prompt_cb)(const char response[]);
 typedef int (*complete_cmd_func)(const char cmd[], void *arg);
 
 /* Initializes command-line mode. */
-void init_cmdline_mode(void);
+void modcline_init(void);
 
 /* Enters command-line editing mode with specified submode.  cmd specifies
  * initial value, ptr is submode-specific data to be passed back. */
-void enter_cmdline_mode(CmdLineSubmode cl_sub_mode, const char cmd[],
-		void *ptr);
+void modcline_enter(CmdLineSubmode cl_sub_mode, const char cmd[], void *ptr);
 
 /* Enters command-line editing mode with prompt submode activated.  cmd
  * specifies initial value, cb is callback called on success, complete is
  * completion function, allow_ee specifies whether issuing external editor is
  * allowed. */
-void enter_prompt_mode(const char prompt[], const char cmd[], prompt_cb cb,
+void modcline_prompt(const char prompt[], const char cmd[], prompt_cb cb,
 		complete_cmd_func complete, int allow_ee);
 
 /* Redraws UI elements of the command-line mode. */
-void redraw_cmdline(void);
+void modcline_redraw(void);
 
 #ifdef TEST
 #include <stddef.h> /* size_t wchar_t */
@@ -111,7 +110,7 @@ typedef struct
 	int old_top;              /* for search_mode */
 	int old_pos;              /* for search_mode */
 	int line_edited;          /* Cache for whether input line changed flag. */
-	int entered_by_mapping;   /* The mode was entered by a mapping. */
+	int enter_mapping_state;  /* The mapping state at entering the mode. */
 	int expanding_abbrev;     /* Abbreviation expansion is in progress. */
 	PromptState state;        /* Prompt state with regard to current input. */
 }
