@@ -22,49 +22,52 @@
 struct view_t;
 
 /* Holds state of a single view mode window. */
-typedef struct view_info_t view_info_t;
+typedef struct modview_info_t modview_info_t;
 
 /* Initializes view mode. */
-void view_init_mode(void);
+void modview_init(void);
 
 /* Enters view mode when possible.  View is the expected output area. */
-void view_enter_mode(struct view_t *view, int explore);
+void modview_enter(struct view_t *view, int explore);
 
 /* Leaves view mode. */
-void view_leave_mode(void);
+void modview_leave(void);
 
 /* Quits view from explore mode.  Assumes the view is not an active one.
  * Automatically redraws view. */
-void view_quit_explore_mode(struct view_t *view);
+void modview_quit_exploring(struct view_t *view);
 
 /* Activates the mode if current pane is in explore mode. */
-void view_try_activate_mode(void);
+void modview_try_activate(void);
 
 /* Performs pre main loop actions for the view mode, which is assumed to be
  * activated. */
-void view_pre(void);
+void modview_pre(void);
 
 /* Performs post main loop actions for the view mode, which is assumed to be
  * activated. */
-void view_post(void);
+void modview_post(void);
 
 /* Displays view mode specific position information.  Assumes that view mode is
  * active. */
-void view_ruler_update(void);
+void modview_ruler_update(void);
 
 /* Redraws view mode. */
-void view_redraw(void);
+void modview_redraw(void);
 
 /* Performs search for the pattern in specified direction inside active view.
  * Returns positive number zero if there is something on the status bar to
  * preserve, otherwise zero is returned. */
-int view_find_pattern(const char pattern[], int backward);
+int modview_find(const char pattern[], int backward);
 
 /* Callback-like routine that handles swapping of panes. */
-void view_panes_swapped(void);
+void modview_panes_swapped(void);
 
 /* Checks whether contents of either view should be updated. */
-void view_check_for_updates(void);
+void modview_check_for_updates(void);
+
+/* Hides graphics that needs special care (doesn't disappear on UI redraw). */
+void modview_hide_graphics(void);
 
 /* Detached views.  These are the views which were either created in detached
  * state or were detached from, but their state (position, etc.) is still
@@ -72,19 +75,19 @@ void view_check_for_updates(void);
 
 /* Creates detached view in the view using output of passed command as a
  * source, but doesn't enter the view mode. */
-void view_detached_make(struct view_t *view, const char cmd[]);
+void modview_detached_make(struct view_t *view, const char cmd[]);
 
 /* Tries to draw an detached view mode and updates internal state if needed.
  * Returns non-zero on success, otherwise zero is returned. */
-int view_detached_draw(void);
+int modview_detached_draw(void);
 
 /* Retrieves viewer command associated with preview created with
- * view_detached_make().  Returns pointer to the viewer command or NULL, if
+ * modview_detached_make().  Returns pointer to the viewer command or NULL, if
  * there is none. */
-const char * view_detached_get_viewer(void);
+const char * modview_detached_get_viewer(void);
 
 /* Frees view info.  The parameter can be NULL. */
-void view_info_free(view_info_t *vi);
+void modview_info_free(modview_info_t *vi);
 
 #endif /* VIFM__MODES__VIEW_H__ */
 
