@@ -35,6 +35,8 @@
 #include "../compat/pthread.h"
 #include "../utils/filter.h"
 #include "../utils/fswatch.h"
+#include "../utils/test_helpers.h"
+#include "../marks.h"
 #include "../status.h"
 #include "../types.h"
 #include "color_scheme.h"
@@ -181,11 +183,13 @@ typedef enum
 }
 NameFormat;
 
+/* Single entry of directory history. */
 typedef struct
 {
-	int rel_pos;
-	char *dir;
-	char *file;
+	char *dir;        /* Directory path. */
+	char *file;       /* File name. */
+	time_t timestamp; /* Time of storing this entry persistently. */
+	int rel_pos;      /* Cursor position from the top. */
 }
 history_t;
 
@@ -273,7 +277,7 @@ struct cv_data_t
 	char *title;
 
 	/* Previous sorting value, before unsorted custom view was loaded. */
-	char sort[SK_COUNT];
+	signed char sort[SK_COUNT];
 
 	/* List of paths that should be ignored (including all nested paths).  Used
 	 * by tree-view. */
@@ -349,11 +353,10 @@ struct view_t
 	cached_entries_t left_column;
 	cached_entries_t right_column;
 
-	/* Monitor that checks for directory changes. */
-	fswatch_t *watch;
-	char watched_dir[PATH_MAX + 1];
+	fswatch_t *watch;  /* Monitor that checks for directory changes. */
+	char *watched_dir; /* Path for which the monitor was created. */
 
-	char last_dir[PATH_MAX + 1];
+	char *last_dir; /* Location visited by the view before the current one. */
 
 	/* Number of files that match current search pattern. */
 	int matches;
@@ -381,13 +384,11 @@ struct view_t
 	int nsaved_selection;   /* Number of items in saved_selection. */
 	char **saved_selection; /* Names of selected files. */
 
-	/* Whether files are selected by user or via range on the command-line.  Say
-	 * some commands (e.g. ga/gA) implement "smart selection", but it shouldn't
-	 * be applied for selected produced by a range. */
-	int user_selection;
+	/* Files were marked for processing, but haven't been processed yet. */
+	int pending_marking;
 
-	int explore_mode;       /* Whether this view is used for file exploring. */
-	struct view_info_t *vi; /* State of explore view (NULL initially). */
+	int explore_mode;          /* Whether this view is used for file exploring. */
+	struct modview_info_t *vi; /* State of explore view (NULL initially). */
 
 	/* Filter which is controlled by user. */
 	struct matcher_t *manual_filter;
@@ -401,8 +402,10 @@ struct view_t
 	 * possible.  Not NULL. */
 	char *prev_auto_filter;
 
+	mark_t special_marks[NUM_SPECIAL_MARKS]; /* View-specific marks. */
+
 	/* List of sorting keys. */
-	char sort[SK_COUNT], sort_g[SK_COUNT];
+	signed char sort[SK_COUNT], sort_g[SK_COUNT];
 	/* Sorting groups (comma-separated list of regular expressions). */
 	char *sort_groups, *sort_groups_g;
 	/* Primary group in compiled form. */
@@ -410,7 +413,7 @@ struct view_t
 
 	int history_num;    /* Number of used history elements. */
 	int history_pos;    /* Current position in history. */
-	history_t *history; /* Directory history itself. */
+	history_t *history; /* Directory history itself (oldest to newest). */
 
 	int local_cs;    /* Whether directory-specific color scheme is in use. */
 	col_scheme_t cs; /* Storage of local (tree-specific) color scheme. */
@@ -459,24 +462,18 @@ struct view_t
 
 extern view_t lwin;
 extern view_t rwin;
-view_t *other_view;
-view_t *curr_view;
+extern view_t *other_view;
+extern view_t *curr_view;
 
-WINDOW *status_bar;
-WINDOW *stat_win;
-WINDOW *job_bar;
-WINDOW *ruler_win;
-WINDOW *input_win;
-WINDOW *menu_win;
-WINDOW *sort_win;
-WINDOW *change_win;
-WINDOW *error_win;
-WINDOW *top_line;
-WINDOW *tab_line;
-
-WINDOW *lborder;
-WINDOW *mborder;
-WINDOW *rborder;
+extern WINDOW *status_bar;
+extern WINDOW *stat_win;
+extern WINDOW *job_bar;
+extern WINDOW *ruler_win;
+extern WINDOW *input_win;
+extern WINDOW *menu_win;
+extern WINDOW *sort_win;
+extern WINDOW *change_win;
+extern WINDOW *error_win;
 
 /* Updates the ruler with information from the view (possibly lazily). */
 void ui_ruler_update(view_t *view, int lazy_redraw);
@@ -516,6 +513,10 @@ int cv_compare(CVType type);
  * non-zero if so, otherwise zero is returned. */
 int cv_tree(CVType type);
 
+/* Resizes all windows according to current screen size and TUI
+ * configuration. */
+void ui_resize_all(void);
+
 /* Redraws whole screen with possible reloading of file lists (depends on
  * argument). */
 void update_screen(UpdateType update_kind);
@@ -648,16 +649,18 @@ void ui_view_title_update(view_t *view);
 
 /* Looks for the given key in sort option.  Returns non-zero when found,
  * otherwise zero is returned. */
-int ui_view_sort_list_contains(const char sort[SK_COUNT], char key);
+int ui_view_sort_list_contains(const signed char sort[SK_COUNT], char key);
 
 /* Ensures that list of sorting keys is sensible (i.e. contains either "name" or
  * "iname" for views, except for unsorted custom view). */
-void ui_view_sort_list_ensure_well_formed(view_t *view, char sort_keys[]);
+void ui_view_sort_list_ensure_well_formed(view_t *view,
+		signed char sort_keys[]);
 
 /* Picks sort array for the view taking custom view into account.  sort should
  * point to sorting array preferred by default.  Returns pointer to the
  * array. */
-char * ui_view_sort_list_get(const view_t *view, const char sort[]);
+signed char * ui_view_sort_list_get(const view_t *view,
+		const signed char sort[]);
 
 /* Checks whether file numbers should be displayed for the view.  Returns
  * non-zero if so, otherwise zero is returned. */
@@ -697,6 +700,10 @@ int ui_qv_height(const view_t *view);
 /* Retrieves width of quickview area.  Returns the width. */
 int ui_qv_width(const view_t *view);
 
+/* If active preview needs special cleanup (i.e., simply redrawing a window
+ * isn't enough. */
+void ui_qv_cleanup_if_needed(void);
+
 /* Invalidates views-specific knowledge about given color scheme. */
 void ui_invalidate_cs(const col_scheme_t *cs);
 
@@ -705,7 +712,7 @@ void ui_invalidate_cs(const col_scheme_t *cs);
 const col_scheme_t * ui_view_get_cs(const view_t *view);
 
 /* Erases view window by filling it with the background color. */
-void ui_view_erase(view_t *view);
+void ui_view_erase(view_t *view, int use_global_cs);
 
 /* Figures out base color for the pane.  Returns the color. */
 col_attr_t ui_get_win_color(const view_t *view, const col_scheme_t *cs);
@@ -758,6 +765,29 @@ void ui_view_redrawn(view_t *view);
  * scheduled event. */
 UiUpdateEvent ui_view_query_scheduled_event(view_t *view);
 
+TSTATIC_DEFS(
+	/* Information for formatting tab title. */
+	typedef struct
+	{
+		char *name;               /* Tab name. */
+		char *num;                /* Tab number. */
+		char *escaped_view_title; /* Auto-formatted title. */
+		char *escaped_path;       /* Current path (could be a file path). */
+		char *cv_title;           /* Prefix of custom view. */
+		int tree;                 /* Whether in tree mode. */
+		int current;              /* Whether it's a current tab. */
+	}
+	tab_title_info_t;
+
+	struct cline_t;
+	struct tab_info_t;
+	typedef char * (*path_func)(const char[]);
+	tab_title_info_t make_tab_title_info(const struct tab_info_t *tab_info,
+		path_func pf, int tab_num, int current_tab);
+	void dispose_tab_title_info(tab_title_info_t *title_info);
+	struct cline_t make_tab_title(const tab_title_info_t *title_info);
+)
+
 #endif /* VIFM__UI__UI_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
