@@ -20,8 +20,6 @@
 #include "fs.h"
 
 #ifdef _WIN32
-#define REQUIRED_WINVER 0x0500
-#include "windefs.h"
 #include <windows.h>
 #include <ntdef.h>
 #include <winioctl.h>
@@ -56,8 +54,6 @@ static int path_exists_internal(const char path[], const char filename[],
 
 #ifndef _WIN32
 static int is_directory(const char path[], int dereference_links);
-#else
-static DWORD win_get_file_attrs(const char path[]);
 #endif
 
 int
@@ -213,43 +209,17 @@ is_symlink(const char path[])
 	struct stat st;
 	return os_lstat(path, &st) == 0 && S_ISLNK(st.st_mode);
 #else
-	char filename[PATH_MAX + 1];
-	DWORD attr;
-	wchar_t *utf16_filename;
-	HANDLE hfind;
-	WIN32_FIND_DATAW ffd;
+	return (win_get_reparse_point_type(path) == IO_REPARSE_TAG_SYMLINK);
+#endif
+}
 
-	attr = win_get_file_attrs(path);
-	if(attr == INVALID_FILE_ATTRIBUTES)
-	{
-		LOG_WERROR(GetLastError());
-		return 0;
-	}
-
-	if(!(attr & FILE_ATTRIBUTE_REPARSE_POINT))
-	{
-		return 0;
-	}
-
-	copy_str(filename, sizeof(filename), path);
-	chosp(filename);
-
-	utf16_filename = utf8_to_utf16(path);
-	hfind = FindFirstFileW(utf16_filename, &ffd);
-	free(utf16_filename);
-
-	if(hfind == INVALID_HANDLE_VALUE)
-	{
-		LOG_WERROR(GetLastError());
-		return 0;
-	}
-
-	if(!FindClose(hfind))
-	{
-		LOG_WERROR(GetLastError());
-	}
-
-	return ffd.dwReserved0 == IO_REPARSE_TAG_SYMLINK;
+int
+is_shortcut(const char path[])
+{
+#ifndef _WIN32
+	return 0;
+#else
+	return (strcasecmp(get_ext(path), "lnk") == 0);
 #endif
 }
 
@@ -322,7 +292,7 @@ get_link_target_abs(const char link[], const char cwd[], char buf[],
 }
 
 int
-get_link_target(const char *link, char *buf, size_t buf_len)
+get_link_target(const char link[], char buf[], size_t buf_len)
 {
 #ifndef _WIN32
 	char *filename;
@@ -348,58 +318,11 @@ get_link_target(const char *link, char *buf, size_t buf_len)
 	buf[len] = '\0';
 	return 0;
 #else
-	char filename[PATH_MAX + 1];
-	DWORD attr;
-	wchar_t *utf16_filename;
-	HANDLE hfile;
-	char rdb[2048];
-	char *t;
-	REPARSE_DATA_BUFFER *sbuf;
-	WCHAR *path;
-
-	if(!is_symlink(link))
+	if(win_symlink_read(link, buf, buf_len) == 0)
 	{
-		return -1;
+		return 0;
 	}
-
-	copy_str(filename, sizeof(filename), link);
-	chosp(filename);
-
-	utf16_filename = utf8_to_utf16(filename);
-	hfile = CreateFileW(utf16_filename, 0, FILE_SHARE_READ | FILE_SHARE_WRITE,
-			NULL, OPEN_EXISTING,
-			FILE_FLAG_BACKUP_SEMANTICS | FILE_FLAG_OPEN_REPARSE_POINT, NULL);
-	free(utf16_filename);
-
-	if(hfile == INVALID_HANDLE_VALUE)
-	{
-		LOG_WERROR(GetLastError());
-		return -1;
-	}
-
-	if(!DeviceIoControl(hfile, FSCTL_GET_REPARSE_POINT, NULL, 0, rdb,
-			sizeof(rdb), &attr, NULL))
-	{
-		LOG_WERROR(GetLastError());
-		CloseHandle(hfile);
-		return -1;
-	}
-	CloseHandle(hfile);
-
-	sbuf = (REPARSE_DATA_BUFFER *)rdb;
-	path = sbuf->SymbolicLinkReparseBuffer.PathBuffer;
-	path[sbuf->SymbolicLinkReparseBuffer.PrintNameOffset/sizeof(WCHAR) +
-			sbuf->SymbolicLinkReparseBuffer.PrintNameLength/sizeof(WCHAR)] = L'\0';
-	t = to_multibyte(path +
-			sbuf->SymbolicLinkReparseBuffer.PrintNameOffset/sizeof(WCHAR));
-	if(strncmp(t, "\\??\\", 4) == 0)
-		strncpy(buf, t + 4, buf_len);
-	else
-		strncpy(buf, t, buf_len);
-	buf[buf_len - 1] = '\0';
-	free(t);
-	system_to_internal_slashes(buf);
-	return 0;
+	return win_shortcut_read(link, buf, buf_len);
 #endif
 }
 
@@ -531,7 +454,7 @@ list_regular_files(const char path[], char *list[], int *len)
 
 		if(is_regular_file(full_path))
 		{
-			*len = add_to_string_array(&list, *len, 1, d->d_name);
+			*len = add_to_string_array(&list, *len, d->d_name);
 		}
 	}
 	os_closedir(dir);
@@ -558,7 +481,7 @@ list_all_files(const char path[], int *len)
 	{
 		if(!is_builtin_dir(d->d_name))
 		{
-			*len = add_to_string_array(&list, *len, 1, d->d_name);
+			*len = add_to_string_array(&list, *len, d->d_name);
 		}
 	}
 	os_closedir(dir);
@@ -906,30 +829,6 @@ is_directory(const char path[], int dereference_links)
 
 #else
 
-/* Obtains attributes of a file.  Skips check for unmounted disks.  Doesn't
- * dereference symbolic links.  Returns the attributes, which is
- * INVALID_FILE_ATTRIBUTES on error. */
-static DWORD
-win_get_file_attrs(const char path[])
-{
-	DWORD attr;
-	wchar_t *utf16_path;
-
-	if(is_path_absolute(path) && !is_unc_path(path))
-	{
-		if(isalpha(path[0]) && !drive_exists(path[0]))
-		{
-			return INVALID_FILE_ATTRIBUTES;
-		}
-	}
-
-	utf16_path = utf8_to_utf16(path);
-	attr = GetFileAttributesW(utf16_path);
-	free(utf16_path);
-
-	return attr;
-}
-
 int
 S_ISLNK(mode_t mode)
 {
