@@ -76,6 +76,10 @@ int paths_are_same(const char s[], const char t[]);
  * it's so, otherwise zero is returned. */
 int is_symlink(const char path[]);
 
+/* Checks whether given path points to a shortcut file (link implemented as a
+ * regular file).  Returns non-zero if it's so, otherwise zero is returned. */
+int is_shortcut(const char path[]);
+
 /* Gets approximate symbolic link type: directory, won't check, anything else.
  * Returns one of SymLinkType values. */
 SymLinkType get_symlink_type(const char path[]);
@@ -87,7 +91,10 @@ SymLinkType get_symlink_type(const char path[]);
 int get_link_target_abs(const char link[], const char cwd[], char buf[],
 		size_t buf_len);
 
-int get_link_target(const char *link, char *buf, size_t buf_len);
+/* Fills the buf of size buf_len with the path symbolic link specified by link
+ * parameter points to.  Returns zero on success, otherwise non-zero is
+ * returned. */
+int get_link_target(const char link[], char buf[], size_t buf_len);
 
 /* Creates directory and all intermediate directories if needed using specified
  * access mode.  If target directory already exists, no error will be raised.
