@@ -86,6 +86,10 @@ int starts_withn(const char str[], const char prefix[], size_t prefix_len);
  * otherwise zero is returned. */
 int ends_with(const char str[], const char suffix[]);
 
+/* Checks whether str ends with the given suffix in a case insensitive way.
+ * Returns non-zero if it's so, otherwise zero is returned. */
+int ends_with_case(const char str[], const char suffix[]);
+
 /* Checks whether the string starts and ends with specified mark characters.
  * Substring can't be empty.  The string can't consist of single mark character
  * when they are equal.  Returns non-zero if so, otherwise zero is returned. */
@@ -266,8 +270,8 @@ int read_int(const char line[], int *i);
  * character. */
 void replace_char(char str[], char from, char to);
 
-/* Splits string on a separator multiple times returning next part.  *state must
- * be NULL for the first call.  Usage example:
+/* Splits string on a separator multiple times returning next part.  Empty items
+ * are skipped.  *state must be NULL for the first call.  Usage example:
  *   char *part = input, *state = NULL;
  *   while((part = split_and_get(part, ':', &state)) != NULL)
  *   {
@@ -275,10 +279,10 @@ void replace_char(char str[], char from, char to);
  *   }
  * Each next step undoes string change made on the previous one, so it could
  * also be (parts of <input> will be added from left to right):
- *   char *prefix = input, *state = NULL;
- *   while((prefix = split_and_get(prefix, ':', &state)) != NULL)
+ *   char *part = input, *state = NULL;
+ *   while((part = split_and_get(part, ':', &state)) != NULL)
  *   {
- *     process <prefix>;
+ *     process <input>;
  *   } */
 char * split_and_get(char str[], char sep, char **state);
 
@@ -293,6 +297,10 @@ int count_lines(const char text[], int max_width);
 /* Counts number of c char occurrences in the s string.  Returns the number. */
 size_t chars_in_str(const char s[], char c);
 
+/* Clones string passed as the argument doubling specified character in the
+ * process.  Returns cloned value. */
+char * double_char(const char str[], char c);
+
 #ifdef _WIN32
 
 /* Same as strstr(), but in case insensitive way. */
