@@ -3,14 +3,14 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <test-utils.h>
+
 #include "../../src/cfg/config.h"
 #include "../../src/ui/ui.h"
 #include "../../src/utils/dynarray.h"
 #include "../../src/utils/str.h"
 #include "../../src/macros.h"
 
-#include "utils.h"
-
 SETUP()
 {
 	update_string(&cfg.shell, "sh");
@@ -32,9 +32,8 @@ SETUP()
 	lwin.dir_entry[3].name = strdup("lfile3");
 	lwin.dir_entry[3].origin = &lwin.curr_dir[0];
 
-	lwin.dir_entry[0].selected = 1;
-	lwin.dir_entry[2].selected = 1;
-	lwin.selected_files = 2;
+	lwin.dir_entry[0].marked = 1;
+	lwin.dir_entry[2].marked = 1;
 
 	/* rwin */
 	strcpy(rwin.curr_dir, "/rwin");
@@ -58,11 +57,10 @@ SETUP()
 	rwin.dir_entry[6].name = strdup("rdir6");
 	rwin.dir_entry[6].origin = &rwin.curr_dir[0];
 
-	rwin.dir_entry[1].selected = 1;
-	rwin.dir_entry[3].selected = 1;
-	rwin.dir_entry[5].selected = 1;
-	rwin.dir_entry[6].selected = 1;
-	rwin.selected_files = 4;
+	rwin.dir_entry[1].marked = 1;
+	rwin.dir_entry[3].marked = 1;
+	rwin.dir_entry[5].marked = 1;
+	rwin.dir_entry[6].marked = 1;
 
 	curr_view = &lwin;
 	other_view = &rwin;
@@ -70,18 +68,11 @@ SETUP()
 
 TEARDOWN()
 {
-	int i;
-
 	update_string(&cfg.shell, NULL);
 	update_string(&cfg.shell_cmd_flag, NULL);
 
-	for(i = 0; i < lwin.list_rows; i++)
-		free(lwin.dir_entry[i].name);
-	dynarray_free(lwin.dir_entry);
-
-	for(i = 0; i < rwin.list_rows; i++)
-		free(rwin.dir_entry[i].name);
-	dynarray_free(rwin.dir_entry);
+	view_teardown(&lwin);
+	view_teardown(&rwin);
 }
 
 TEST(f)
